#include "sysuserv1thread.h"
#include <topcore/topclassabs.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>

SysUserV1Thread::SysUserV1Thread(QObject *iParent)
    : TopClassThreadAbs(iParent)
{

}
SysUserV1Thread::~SysUserV1Thread()
{

}
void SysUserV1Thread::run()
{
    if (invokeName() == "LOAD_DATA") {
        loadData(invokeParameter().toInt());
    } else if (invokeName() == "SAVE_DATA") {
        saveData(invokeParameter().toMap());
    }
}

//iIdInt  invokeParameter().toInt()
void SysUserV1Thread::loadData(int iIdInt)
{
    //dataRes传回给QVariant data = doThreadWork(new SysUserV1Thread(this), "LOAD_DATA", QVariant(this->uid()))定义的data对象
    TDataResponse dataRes;
    //TSql这个类执行sql语句，获取一个数据库，这个数据库在软件里选择
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin(); // 开始操作数据库
    try {
        // Selector：查
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_user AS U LEFT JOIN pub_contacts AS C ON U.contcat_id = C.id");
        sqlSelector.setField("U.*, C.mail");
        sqlSelector.setWhere("U.id", QVariant(iIdInt));
        QVariantMap dataMap = sqlQuery.selectMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }
        sqlQuery.commit();
        dataRes.setData(dataMap);
        setInvokeResult(dataRes.toVariantMap());
        return;
        //捕获异常
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysUserV1Thread::saveData(const QVariantMap &iDataMap)
{
    TDataResponse dataRes;
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        //判断是否重复
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_user");
        sqlSelector.setField("COUNT(1)");
        sqlSelector.addWhere("username", iDataMap.value("username"));
        sqlSelector.addWhere("id", iDataMap.value("id"), "!=");
        int count = sqlQuery.selectCount(sqlSelector);
        if (count > 0) {
            throw TError(ttr("'%1' already exists!").arg(iDataMap.value("username").toString()), "ERROR", "ALREADY_EXISTS");
        }

        // 更新pub_contacts表的mail
        QVariantMap contactMap;
        contactMap.insert("mail", iDataMap.value("mail"));
        QString username = iDataMap.value("username").toString();
        QString fullname = iDataMap.value("fullname").toString();
        QString name;
        if (!fullname.isEmpty() && (username != fullname)) {
            name = QString("%1(%2)").arg(fullname).arg(username);
        } else {
            name = username;
        }
        contactMap.insert("name", name);
        contactMap.insert("id", iDataMap.value("contcat_id").toInt());
        contactMap.insert("status", iDataMap.value("status").toString() == "active" ? "active" : "inactive");
        QStringList contactFields = contactMap.keys();
        contactFields.removeOne("id");

        TSqlInserterV2 sqlInserter;
        sqlInserter.setTable("pub_contacts");
        sqlInserter.setData(contactMap);
        sqlInserter.setField(contactFields);
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");

        QVariant contactId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }
        // 更新sys_user表
        QVariantMap dataMap = iDataMap;
        QStringList fieldLst = dataMap.keys();
        dataMap.insert("contcat_id", contactId);
        if (iDataMap.value("id").toInt() == 0) {
            // 新建时插入默认密码
            QString password = iDataMap.value("password").toString();
            if (password.isEmpty()) {
                password = QStringLiteral("123456");
            }
            dataMap.insert("password", TDataParse::getVariantMd5(password));
        } else {
            fieldLst.removeOne("password");
        }
        fieldLst.removeOne("id");
        fieldLst.removeOne("mail");
        sqlInserter.clear();
        sqlInserter.setTable("sys_user");
        sqlInserter.setData(dataMap);
        sqlInserter.setField(fieldLst);
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");

        QVariant userId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }
        sqlQuery.commit();
        dataRes.setData(userId);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}
